from typing import Literal
from libqtile.lazy import lazy
from qtile_extras.popup.toolkit import PopupRelativeLayout, PopupImage, PopupText


from qtile_lxa.widget.ThemeManager.Config import colors
from qtile_lxa.widget.ThemeManager.Config import ThemeManagerConfig
from qtile_lxa import __DEFAULTS__, __BASE_DIR__, __ASSETS_DIR__

theme_config = ThemeManagerConfig().load_config()
color_scheme = colors.get_color_scheme(
    theme_config.get("color", {}).get("scheme", "dark_pl")
)
active_color = colors.rgba(color_scheme["active"], 0.4)
inactive_color = colors.rgba(color_scheme["inactive"], 0.4)


class PowerMenu:
    menu_instance = None

    def __init__(
        self,
        qtile,
        sleep_mode: str = __DEFAULTS__.power_menu.sleep_mode,
        screenlock_effect: str = __DEFAULTS__.theme_manager.pywall.screenlock_effect,
    ):
        self.qtile = qtile
        self.sleep_mode = sleep_mode
        self.screenlock_effect = screenlock_effect
        self.controls = []
        self.layout = None
        self.create_controls()

    def create_controls(self):
        sleep_commands = {
            "sleep": "systemctl suspend",
            "hibernate": "systemctl hibernate",
            "hybrid-sleep": "systemctl hybrid-sleep",
        }
        sleep_command = sleep_commands.get(self.sleep_mode, sleep_commands["sleep"])

        menu_items = [
            (
                "lock.png",
                "Lock",
                lazy.spawn(f"betterlockscreen -l {self.screenlock_effect}"),
            ),
            ("logout.png", "Logout", lazy.shutdown()),
            ("sleep.png", self.sleep_mode.capitalize(), lazy.spawn(sleep_command)),
            ("restart.png", "Restart", lazy.spawn("systemctl reboot")),
            ("shutdown.png", "Shutdown", lazy.spawn("systemctl poweroff")),
        ]

        for i, (icon, label, action) in enumerate(menu_items):
            pos_x = 0.05 + i * 0.2
            self.controls.append(
                PopupImage(
                    filename=str(__ASSETS_DIR__ / f"icons/{icon}"),
                    pos_x=pos_x,
                    pos_y=0.2,
                    width=0.1,
                    height=0.5,
                    highlight_radius=35,
                    highlight_method="border",
                    highlight=color_scheme["active"],
                    mouse_callbacks={"Button1": action},
                )
            )
            self.controls.append(
                PopupText(
                    text=label,
                    pos_x=pos_x,
                    pos_y=0.75,
                    width=0.1,
                    height=0.2,
                    h_align="center",
                )
            )

        # "Close" button
        self.controls.append(
            PopupImage(
                filename=str(__ASSETS_DIR__ / "icons/close.png"),
                pos_x=0.95,
                pos_y=0.01,
                width=0.02,
                height=0.09,
                highlight_radius=10,
                highlight="D91656",
                mouse_callbacks={"Button1": self.hide},
            )
        )

    def show(self):
        if PowerMenu.menu_instance:
            return

        self.layout = PopupRelativeLayout(
            self.qtile,
            width=1000,
            height=250,
            controls=self.controls,
            background=inactive_color,
            border=active_color,
            border_width=2,
            initial_focus=None,
            close_on_click=True,
            hide_on_mouse_leave=True,
        )
        self.layout.show(centered=True)
        PowerMenu.menu_instance = self

    def hide(self):
        """Hide the PowerMenu."""
        if self.layout:
            self.layout.hide()
            self.layout = None
            PowerMenu.menu_instance = None


def show_power_menu(qtile, sleep_mode=__DEFAULTS__.power_menu.sleep_mode):
    if not PowerMenu.menu_instance:
        menu = PowerMenu(qtile, sleep_mode)
        menu.show()
    else:
        PowerMenu.menu_instance.hide()
        menu = PowerMenu(qtile, sleep_mode)
        menu.show()
